@@ -6,7 +6,9 @@ import nb2workflow.nbadapter as nbadapter
 
 logger = logging.getLogger(__name__)
 
+import requests
 import rdflib
+import odakb.sparql
 
 try:
     import owlready2
@@ -14,9 +16,15 @@ try:
     xsd = owlready2.get_ontology("https://www.w3.org/2001/XMLSchema#").load()
     kees = owlready2.get_ontology("http://linkeddata.center/kees/v1#").load()
 
-    fno = owlready2.get_ontology("http://ontology.odahub.io/function.rdf").load()
-    fno.base_iri="https://w3id.org/function/ontology#"
-except:
+    G = rdflib.Graph()
+    open("function.xml", "w").write(G.load(io.StringIO(requests.get("https://raw.githubusercontent.com/FnOio/fnoio.github.io/master/ontology/0.4.1/function.ttl").text), format="turtle").serialize(format="xml"))
+
+    fno = owlready2.get_ontology("function.xml").load()
+    # fno.base_iri="https://w3id.org/function/ontology#"
+
+    odaworkflow = owlready2.get_ontology("http://odahub.io/ontology/workflow#")
+except Exception as e:
+    logger.warning('unable to import owlready2: %s', e)
     owlready2=None
 
 def get_dda():
@@ -45,36 +53,57 @@ def to_xsd_type(p):
     return p.get('owl_type',"http://www.w3.org/2001/XMLSchema#"+out_type)
 
 
+#TODO: return owl option as an option
 
-def function_semantic_signature(dda, function_name, parameters, output):
-    if owlready2 is None:
-        return
-    with dda:
-        parameter_attrs={}
-        for pn,pv in parameters.items():
-            p_cls = type(
-                        pn,(fno.Parameter,),
-                        {}
-                    )
-            parameter_attrs[pn] = p_cls
-
-            s,p,o = (dda.graph.abbreviate(p_cls.iri), 
-                     dda.graph.abbreviate(fno.type.iri),
-                     dda.graph.abbreviate(to_xsd_type(pv)))
-
-            if len(dda.get_triples(s,p,o)) == 0:
-                dda.add_triple(s,p,o)
-
-
-        cls = type(function_name,(dda.WebDataAnalysis,),
-                    dict(expects=parameter_attrs.values())
-                )
-            
-        cls().url = "http://api.odahub.io/"+function_name
-
+def function_semantic_signature(function_name, location, parameters, output):
+    G = rdflib.Graph()
     
+    wfl = rdflib.URIRef(f'http://odahub.io/workflows#{function_name}')    
+    
+    oda_ns = rdflib.Namespace('http://odahub.io/ontology#')
+    rdf_ns = rdflib.Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#')
+    wfl_p_ns = rdflib.Namespace(f'http://odahub.io/workflows/{function_name}/parameter_bindings#')
+    
+    G.bind('oda', oda_ns)
+    G.bind('rdfs', rdf_ns)
+    # G.bind('wfl', wfl_p_ns)
 
-def service_semantic_signature(nbas, format="rdfxml"):
+    G.add((wfl, rdf_ns['type'], oda_ns['workflow']))
+    G.add((wfl, oda_ns['location'], rdflib.Literal(location)))
+
+    for pn, pv in parameters.items():
+        p_uri = wfl_p_ns[pn]
+        G.add((p_uri, rdf_ns['type'], rdflib.URIRef(to_xsd_type(pv))))
+        G.add((wfl, oda_ns['expects'], p_uri))
+                    
+    return G
+
+
+def service_semantic_signature(nbas, format="xml") -> str:
+    G = rdflib.Graph()
+
+    for target, nba in nbas.items():
+        logger.info("target: %s nba: %s", target, nba)
+        S_G = function_semantic_signature(
+                                    nba.unique_name,
+                                    location=nba.notebook_origin,
+                                    parameters=nba.extract_parameters(),
+                                    output=nba.extract_output_declarations())
+
+        for t in S_G:
+            G.add(t)
+
+
+    rdf_str = G.serialize(format=format)
+
+    logger.debug(rdf_str)
+    
+    return rdf_str
+
+
+
+
+def service_semantic_signature_owl(nbas, format="rdfxml"):
     if owlready2 is None:
         return
     dda = get_dda()
@@ -106,28 +135,21 @@ def service_semantic_signature(nbas, format="rdfxml"):
 
 
 
-def nb2rdf(notebook_fn, rdf_fn):
-    if owlready2 is None:
-        return
+def nb2rdf(notebook_fn: str) -> str:
     nba = nbadapter.NotebookAdapter(notebook_fn)
 
-    with open(rdf_fn, "wb") as f:
-        rdf = service_semantic_signature(dict(local=nba))
+    rdf = service_semantic_signature(dict(local=nba), format="turtle")
+            
+    logging.getLogger().info("rdf: %s", rdf)
 
-        G = rdflib.Graph()
-        G.parse(data=rdf, format="xml")
-        rdf = G.serialize(format="turtle")
-
-        logging.getLogger().info("rdf: "+rdf.decode())
-        f.write(rdf)
+    return rdf
 
 
 def main():
     parser = argparse.ArgumentParser(description='Process some integers.')
     parser.add_argument('notebook', metavar='notebook', type=str)
-    parser.add_argument('rdf', metavar='rdf', type=str)
-    parser.add_argument('--publish', metavar='upstream-url', type=str, default=None)
-    parser.add_argument('--publish-as', metavar='published url', type=str, default=None)
+    parser.add_argument('--out-rdf', metavar='rdf', type=str)
+    parser.add_argument('--publish', action="store_true")
     parser.add_argument('--debug', action="store_true")
 
     args = parser.parse_args()
@@ -149,7 +171,20 @@ def main():
         root.setLevel(logging.INFO)
         handler.setLevel(logging.INFO)
 
-    nb2rdf(args.notebook, args.rdf)
+    rdf = nb2rdf(args.notebook)
+
+    if args.out_rdf:
+        with open(args.out_rdf, "wt") as f:
+            f.write(rdf)
+
+    if args.publish:
+        G = rdflib.Graph()
+        G.parse(data=rdf)
+
+        odakb.sparql.insert("\n".join([
+            f"{s.n3()} {p.n3()} {o.n3()} ." for s, p, o in G
+        ]))
+    
 
 
 if __name__ == "__main__":
